@@ -1,10 +1,13 @@
 import copy
+from os import times
 from sys import getswitchinterval, exit
 
+
 from jsonrpcserver import async_dispatch
 import json
 import asyncio
 import websockets
+import hashlib
 import aiohttp
 from aiohttp import web
 
@@ -14,6 +17,11 @@ from .meter.account import (
     solo,
     keystore as _keystore,
 )
+
+from .utils.types import (
+
+    encode_number
+)
 from .meter.client import meter
 import requests
 import click
@@ -105,9 +113,84 @@ async def handleRequest(request, logging=False, debug=False):
         return web.Response(headers=res_headers, content_type="text/plain").text
 
 
+BLOCK_FORMATTERS = {
+   
+   
+    "timestamp": encode_number,
+    "gasLimit": encode_number,
+    "gasUsed": encode_number,
+    "epoch":encode_number,
+    "k":encode_number
+    
+}
+
+
+
+def meter_block_convert_to_eth_block(block):
+    # sha3Uncles, logsBloom, difficaulty, extraData are the required fields. nonce is optional
+    n = block["nonce"]
+    if n == 0:
+        block["nonce"] = '0x0000000000000000'
+    else:
+        block["nonce"] = encode_number(n, 8)
+
+    # TODO: fix "fake" sha3Uncles
+    block['sha3Uncles'] = '0x1dcc4de8dec75d7aab85b567b6ccd41ad312451b948a7413f0a142fd40d49347'
+    # TODO: fix "fake" transactions root
+    if len(block['transactions']) ==0:
+        block['transactionsRoot'] = '0x56e81f171bcc55a6ff8345e692c0f86e5b48e01b996cadc001622fb5e363b421'
+    #block['logsBloom'] = '0x00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000'
+    block['difficulty'] = '0x'
+    block['extraData'] = '0x'
+    if 'kblockData' in block:
+        del block['kblockData']
+    if 'powBlocks' in block:
+        del block['powBlocks']
+    for key, value in block.items():
+        if key in BLOCK_FORMATTERS:
+           block[key] =  encode_number(value).decode()
+    return block
+
+    
+
+conns = {}
+WSURL_NHEADS = 'ws://127.0.0.1:8669/subscriptions/beat'
+
+def hash_digest(param):
+    h = hashlib.sha224(param.encode())
+    digest = h.hexdigest()
+    return digest
+
+async def run_ws_client(endpoint):
+    ws_endpoint = endpoint.replace('https', 'ws').replace('http','ws')+'/subscriptions/beat'
+    async with websockets.connect(ws_endpoint) as beatWS:
+        async for msg in beatWS:
+            # print('got: ', msg)
+            for key in list(conns.keys()):
+                ws = conns[key]
+                r = json.loads(msg)
+                if r.get("number"):
+                    num = int(r.get("number"), 16)
+                    print("forward block %d to conn %s" %(num,key))
+                else:
+                    print('forward to conn', key)
+
+                r['timestamp']  = hex(r['timestamp'])
+                r['gasLimit'] = hex(r['gasLimit']).replace('0x0', '')
+                r['gasUsed'] = hex(r['gasUsed']).replace('0x0','')
+                r['nonce'] = hex(r['nonce']).replace('0x0', '0x0000000000000000')
+                try:
+                    out = json.dumps({"jsonrpc": "2.0", "method":"eth_subscription" ,"params":{"subscription":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827", "result":r},})
+                    await ws.send_str(out)
+                except Exception as e:
+                    del conns[key]
+                    print('error happend for ', key, 'ignored', e)
+
+
 async def websocket_handler(request):
         count = 1
         headers = request.headers
+        # print("HEADERS: ", headers)
         if (
             headers.get("connection", "").lower() == "upgrade"
             and headers.get("upgrade", "").lower() == "websocket"
@@ -117,33 +200,62 @@ async def websocket_handler(request):
             ws = web.WebSocketResponse()
             try:
                  await ws.prepare(request)
+                 key = request.headers.get("sec-websocket-key", "")
                  async for msg in ws:
-                    if msg.type == aiohttp.WSMsgType.TEXT and msg.data != '\n' and msg.data.strip():
-                        if(json.loads(msg.data)['method'] == "eth_subscribe"):
-                            #send a subscription id to the client
-                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827", "id":count}))
-                            count = count + 1
-                            #begin subscription
-                            while True:
-                               
-                                res = await handleRequest( json.loads(msg.data), False, False)
-                                copy_obj = copy.deepcopy(json.loads(res))
-                                copy_obj["result"]["timestamp"] = hex(copy_obj["result"]["timestamp"])
-                                copy_obj["result"]["k"] = hex(copy_obj["result"]["k"])
-                                copy_obj["result"]["gasLimit"] = hex(copy_obj["result"]["gasLimit"])
-                                copy_obj["result"]["gasUsed"] = hex(copy_obj["result"]["gasUsed"])
-                                copy_obj["result"]["nonce"] = hex(copy_obj["result"]["nonce"])
-                                copy_obj["result"]["epoch"] = hex(copy_obj["result"]["epoch"])
+                    # print("REQ: ", msg.data)
+                    if msg.type == aiohttp.WSMsgType.TEXT and msg.data.strip():
+                        # if is a valid json request
 
-                                # convert the subscription object into an appropriate response
-                                res_obj = {"jsonrpc": copy_obj["jsonrpc"] , "method":"eth_subscription", "params":{"result":copy_obj["result"], "subscription":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827"}}
-                                await ws.send_str(json.dumps(res_obj))
-                        elif (json.loads(msg.data)['method'] == "eth_unsubscribe"):
-                            # return 'true' for eth_unsubscribe
-                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":True, "id":count}))
+                        jreq = json.loads(msg.data)
+
+                        # handle batch requests
+                        if isinstance(jreq, list):
+                            print("HANDLE LIST")
+                            ress = []
+                            for r in jreq:
+                                res = await handleRequest(r, False, False)
+                                ress.append(json.loads(res))
+                            await ws.send_str(json.dumps(ress))
+                            continue
+
+                        if 'method' not in jreq:
+                            # not a valid json request
+                            continue
+
+                        if(jreq['method'] == "eth_subscribe"):
+                            # handle subscribe
+                            print("SUBSCRIBE:", msg.data, "key:", key)
+                            if key in conns:
+                                continue
+                            conns[key] = ws
+                            print("appended %s to conns" % key)
+                            #send a subscription id to the client
+                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827", "id":jreq['id']}))
+                            
+                            #begin subscription
+                            # while True:
+                            
+                            #     res = await handleRequest( json.loads(msg.data), False, False)
+                            #     copy_obj = copy.deepcopy(json.loads(res))
+                            #     # convert the subscription object into an appropriate response
+                            #     result = meter_block_convert_to_eth_block(copy_obj['result'])
+                                
+                            #     res_obj = {"jsonrpc": copy_obj["jsonrpc"] , "method":"eth_subscription", "params":{"result":result, "subscription":"0x00640404976e52864c3cfd120e5cc28aac3f644748ee6e8be185fb780cdfd827"}}
+                            #     await ws.send_str(json.dumps(res_obj))
+                        elif (jreq['method'] == "eth_unsubscribe"):
+                            # handle unsubscribe
+                            await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":True, "id":jreq['id']}))
+                            if key not in conns:
+                                continue
+                            del conns[key]
+                            print("UNSUBSCRIBE key:", key)
+                            await ws.close()
                         else:
-                            res = await handleRequest( json.loads(msg.data), False, False)
+                            # handle normal requests
+                            res = await handleRequest(json.loads(msg.data), False, False)
+                            print("Send to %s" % key)
                             await ws.send_str(res)
+                            # await ws.send_str(json.dumps({"jsonrpc":"2.0", "result":json.loads(res), "id":count}))
                         
                     elif msg.type == aiohttp.WSMsgType.BINARY:
                         await ws.send_str(msg.data)
@@ -155,14 +267,21 @@ async def websocket_handler(request):
                         await ws.pong()
                         
                     elif ws.closed:
+                        if key not in conns:
+                            return
+                        del conns[key]
                         await ws.close(code=ws.close_code, message=msg.extra)
                     else:
-                        await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":"", "id":count}))
+                        print("unknown REQ: ", msg)
+                        pass
+                        # await ws.send_str(json.dumps({"jsonrpc": "2.0" ,"result":"", "id":count}))
             
-            except:
+            except Exception as e:
+                print("ERROR HAPPENED:", e)
                 await ws.close()
         else:
-            return await handleRequest(request, False, False)
+            # return await handleRequest(request, False, False)
+            pass
            
 
 def get_http_app(host, port, endpoint, keystore, passcode, log, debug, chainid):
@@ -243,6 +362,8 @@ async def run_server(host, port, endpoint, keystore, passcode, log, debug, chain
     await ws.start()
     print("Websocket Listening on %s:%s" % (host, int(port)+1))
 
+    ws_client = asyncio.create_task(run_ws_client(endpoint))
+    await ws_client
     while True:
         await asyncio.sleep(3600)  # sleep forever
 
