@@ -4,7 +4,7 @@ import json
 async def post(endpoint_uri, data, **kwargs):
     async with aiohttp.ClientSession() as session:
         async with session.post(endpoint_uri, json=data, **kwargs) as response:
-            print("Raw response from server: ", response.status, await response.text())
+            # print("Raw response from server: ", response.status, await response.text())
             ctype = response.headers['Content-Type']
             expect = "text/plain"
             if expect in ctype:
@@ -17,7 +17,7 @@ async def post(endpoint_uri, data, **kwargs):
 async def get(endpoint_uri, params, **kwargs):
     async with aiohttp.ClientSession() as session:
         async with session.get(endpoint_uri, params=params, **kwargs) as response:
-            print("Raw response from server: ", response.status,  await response.text())
+            # print("Raw response from server: ", response.status,  await response.text())
             ctype = response.headers['Content-Type']
             expect = "text/plain"
             if expect in ctype:
